﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.IO;
using System.Windows.Forms;
using System.Net.Mail;
using System.Collections.Specialized;
using System.Net.Mime;


namespace EmailParserScriptingSDK.DoNotModifyThis
{
    public partial class EmailEditor : UserControl
    {

        // The path where the used email is saved
        public String file_path_saved_email = System.Windows.Forms.Application.UserAppDataPath + "\\EmailToTest.xml";

        /// <summary>
        /// This component simply shows an email editor
        /// The constructor takes an email object that will be shown
        /// </summary>
        /// <param name="fill_with_this_email"></param>
        public EmailEditor()
        {
            InitializeComponent();

            LoadEmail();
        }


        ~EmailEditor()
        {
            SaveEmail();
        }


        /// <summary>
        /// Returns the email as a NameValueCollection
        /// </summary>
        /// <returns></returns>
        public NameValueCollection GetEmailAsNameValueCollection()
        {
            NameValueCollection returned = new NameValueCollection();


            try
            {
                returned.Add("From", textBoxFrom.Text);

            }
            catch
            {
                ControlFlasher.Start(textBoxFrom, 5, 250, Color.FromArgb(243, 132, 132), "");
                return (null);
            }



            try
            {

                returned.Add("To", textBoxTo.Text);
            }
            catch
            {
                ControlFlasher.Start(textBoxTo, 5, 250, Color.FromArgb(243, 132, 132), "");
                return (null);
            }



            try
            {

                returned.Add("CC", textBoxCC.Text);
            }
            catch
            {
                ControlFlasher.Start(textBoxCC, 5, 250, Color.FromArgb(243, 132, 132), "");
                return (null);
            }



            try
            {

                returned.Add("BCC", textBoxBCC.Text);
            }
            catch
            {
                ControlFlasher.Start(textBoxBCC, 5, 250, Color.FromArgb(243, 132, 132), "");
                return (null);
            }


            try
            {
                returned.Add("Subject", textBoxSubject.Text);
            }
            catch
            {
                ControlFlasher.Start(textBoxSubject, 5, 250, Color.FromArgb(243, 132, 132), "");
                return (null);
            }



            try
            {

                returned.Add("Body", textBoxBody.Text);
            }
            catch
            {
                ControlFlasher.Start(textBoxBody, 5, 250, Color.FromArgb(243, 132, 132), "");
                return (null);
            }



            try
            {

                returned.Add("Body_HTML", textBoxBodyHTML.Text);
            }
            catch
            {
                ControlFlasher.Start(textBoxBodyHTML, 5, 250, Color.FromArgb(243, 132, 132), "");
                return (null);
            }



            try
            {
                returned.Add("DateTimeSent", dateTimePicker_datetimereceived.Value.ToString());
            }
            catch
            {
                ControlFlasher.Start(dateTimePicker_datetimereceived, 5, 250, Color.FromArgb(243, 132, 132), "");
                return (null);
            }



            try
            {
                returned.Add("DateTimeReceived", dateTimePicker_datetimesent.Value.ToString());
            }
            catch
            {
                ControlFlasher.Start(dateTimePicker_datetimesent, 5, 250, Color.FromArgb(243, 132, 132), "");
                return (null);


            }

            return (returned);



        }


        /// <summary>
        /// Returns the email as a MailMessage object
        /// </summary>
        /// <returns></returns>
        public MailMessage GetEmailAsMailMessage()
        {
            MailMessage returned = new MailMessage();

            try
            {
                returned.From = new MailAddress(textBoxFrom.Text);
            }
            catch
            {
                ControlFlasher.Start(textBoxFrom, 5, 250, Color.FromArgb(243, 132, 132), textBoxFrom.Text);
                return (null);
            }


            try
            {
                string[] to_field_addresses = textBoxTo.Text.Split(';');
                foreach (string to_field_address in to_field_addresses)
                    returned.To.Add(new MailAddress(to_field_address));
            }
            catch
            {
                ControlFlasher.Start(textBoxTo, 5, 250, Color.FromArgb(243, 132, 132), textBoxTo.Text);
                return (null);
            }


            try
            {
                if (textBoxCC.Text.Length > 0)
                {
                    string[] cc_field_addresses = textBoxCC.Text.Split(';');
                    foreach (string cc_field_address in cc_field_addresses)
                        returned.To.Add(new MailAddress(cc_field_address));
                }
            }
            catch
            {
                ControlFlasher.Start(textBoxCC, 5, 250, Color.FromArgb(243, 132, 132), textBoxCC.Text);
                return (null);
            }

            try
            {
                if (textBoxBCC.Text.Length > 0)
                {
                    string[] bcc_field_addresses = textBoxBCC.Text.Split(';');
                    foreach (string bcc_field_address in bcc_field_addresses)
                        returned.To.Add(new MailAddress(bcc_field_address));
                }
            }
            catch
            {
                ControlFlasher.Start(textBoxBCC, 5, 250, Color.FromArgb(243, 132, 132), textBoxBCC.Text);
                return (null);
            }


            try
            {
                foreach (String filename in listBoxAttachments.Items)
                {
                    Attachment attachment = new Attachment(filename);
                    returned.Attachments.Add(attachment);
                }
            }
            catch
            {
                ControlFlasher.Start(listBoxAttachments, 5, 250, Color.FromArgb(243, 132, 132), "");
                return (null);
            }

            returned.Subject = textBoxSubject.Text;
            returned.Body = textBoxBody.Text;

            // Add the alternate HTML body to the message.
            ContentType mimeType = new System.Net.Mime.ContentType("text/html");
            AlternateView alternate = AlternateView.CreateAlternateViewFromString(textBoxBodyHTML.Text, mimeType);
            returned.AlternateViews.Add(alternate);







            return (returned);



        }




        /// <summary>
        /// Saves the email to user profile
        /// </summary>
        public void SaveEmail()
        {
            SerializableEmail email = new SerializableEmail();

            email.From = textBoxFrom.Text;
            email.To = textBoxTo.Text;
            email.CC = textBoxCC.Text;
            email.BCC = textBoxBCC.Text;
            email.Subject = textBoxSubject.Text;
            email.BodyHTML = textBoxBodyHTML.Text;
            email.BodyPlainText = textBoxBody.Text;

            foreach (Object item in listBoxAttachments.Items)
            {
                email.Attachments.Add(item.ToString());
            }

            try
            {

                String xml_serialized_email = email.Serialize();

                // Create the file or truncate the existing
                FileStream file = File.Open(file_path_saved_email, FileMode.Create);
                file.Close();

                StreamWriter stream_writer = File.CreateText(System.Windows.Forms.Application.UserAppDataPath + "\\EmailToTest.xml");
                stream_writer.Write(xml_serialized_email);
                stream_writer.Close();
            }
            catch (Exception e)
            {
                MessageBox.Show(e.Message);
            }
        }




        /// <summary>
        /// Loads the email from the user profile
        /// </summary>
        public void LoadEmail()
        {
            // If the file does not exist do nothing
            if (!File.Exists(file_path_saved_email))
                return;


            try
            {
                SerializableEmail email = new SerializableEmail();
                StreamReader stream_reader = new StreamReader(file_path_saved_email);

                email.Unserialize(stream_reader.ReadToEnd());

                stream_reader.Close();

                textBoxFrom.Text = email.From;
                textBoxTo.Text = email.To;
                textBoxCC.Text = email.CC;
                textBoxBCC.Text = email.BCC;
                textBoxSubject.Text = email.Subject;
                textBoxBodyHTML.Text = email.BodyHTML.Replace("\n","\r\n");
                textBoxBody.Text = email.BodyPlainText.Replace("\n", "\r\n");

                foreach (String attachment in email.Attachments)
                {
                    listBoxAttachments.Items.Add(attachment);
                }

            }
            catch (Exception e)
            {
                MessageBox.Show(e.Message);
            }

        }




        private void buttonAddAttachment_Click_1(object sender, EventArgs e)
        {
            //Open file file dialog box
            OpenFileDialog LoadFileDialog1 = new OpenFileDialog();
            LoadFileDialog1.Filter = "All Files (*.*)|*.*";
            LoadFileDialog1.FilterIndex = 1;
            LoadFileDialog1.RestoreDirectory = true;
            LoadFileDialog1.CheckFileExists = false;
            LoadFileDialog1.Title = "Select file to attach";
            LoadFileDialog1.Multiselect = true;

            if (LoadFileDialog1.ShowDialog() == DialogResult.OK)
            {
                foreach (String filename in LoadFileDialog1.FileNames)
                    listBoxAttachments.Items.Add(filename);
            }
        }

        private void buttonRemoveAttachment_Click(object sender, EventArgs e)
        {
            List<Object> items_to_delete = new List<Object>();

            foreach (Object item in listBoxAttachments.SelectedItems)
                items_to_delete.Add(item);

            foreach (Object item in items_to_delete)
                listBoxAttachments.Items.Remove(item);
        }







    }
}
