﻿using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Net.Mail;
using System.Linq;
using System.Reflection;
using System.CodeDom.Compiler;
using System.Text.RegularExpressions;
using System.IO;



namespace EmailParserScriptingSDK
{
    public partial class TestEmailParserDialogRun : Form
    {
        private BackgroundWorker backgroundWorkerTestingEmailFilter;

        // This delegate enables asynchronous calls for setting
        // the text property on a TextBox control.
        delegate void ShowMessageCallback(string text);

        string input_text_used_for_testing;

        /// <summary>
        /// Constructor
        /// </summary>
        public TestEmailParserDialogRun(string input_text_param)
        {
            InitializeComponent();

            input_text_used_for_testing = input_text_param;

            backgroundWorkerTestingEmailFilter = new BackgroundWorker();
            backgroundWorkerTestingEmailFilter.DoWork += TestingWorkerProc;
            backgroundWorkerTestingEmailFilter.RunWorkerCompleted += TestingWorkerFinished;

            // Lanzar el thread del test cuando se haya cargado la ventana
            Application.Idle += new EventHandler(OnLoaded);
        }

        private void buttonClose_Click(object sender, EventArgs e)
        {
            this.Close();
        }




        // Para lanzar el thread del test. No se hace en el constructor pq
        // lanza el thread antes de que la ventana esté completamente cargada
        private void OnLoaded(object sender, EventArgs args)
        {
            Application.Idle -= new EventHandler(OnLoaded);
            backgroundWorkerTestingEmailFilter.RunWorkerAsync(input_text_used_for_testing);
        }



        /// <summary>
        /// Thread that runs the test
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void TestingWorkerProc(object sender, DoWorkEventArgs e)
        {
            try
            {
                // Check the parameter
                if (e.Argument == null || e.Argument.GetType() != typeof(string))
                    return;

                string input_text = e.Argument as string;

                MyScriptBasedEmailParser parser_to_test = new MyScriptBasedEmailParser();
                parser_to_test.Print = ShowMessage;
                List<string> results = parser_to_test.ExtractTextFrom(input_text);

                ShowMessage("The email parser has produced the following results:");
                ShowMessage("");

                foreach (String result in results)
                    ShowMessage(result);


                
            }
            catch(Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }





        /// <summary>
        /// Shows a message in the test window
        /// </summary>
        /// <param name="message"></param>
        public void ShowMessage(String message)
        {
            // Thread safe, check if this method is called
            // from the same thread that created the richtextbox control
            if (textBoxProgress.InvokeRequired)
            {
                // It's on a different thread, so use Invoke.
                ShowMessageCallback cb = new ShowMessageCallback(ShowMessage);
                textBoxProgress.Invoke(cb, new object[] { message });
            }
            else
            {
                // It's on the same thread, no need for Invoke
                textBoxProgress.AppendText(message + "\r\n");
            }
        }


        /// <summary>
        /// The testing thread has finished. Close the window
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void TestingWorkerFinished(object sender, RunWorkerCompletedEventArgs e)
        {
            ShowMessage("Finished");
        }


    }
}
